@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2016 The FlameRobin Development Team
+  Copyright (c) 2004-2022 The FlameRobin Development Team
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -51,6 +51,7 @@
 #include "metadata/exception.h"
 #include "metadata/function.h"
 #include "metadata/generator.h"
+#include "metadata/Index.h"
 #include "metadata/parameter.h"
 #include "metadata/package.h"
 #include "metadata/procedure.h"
@@ -72,6 +73,7 @@ private:
     bool showColumnsM;
     int showComputedM;
     int showDomainsM;
+    bool showSystemIndicesM;
     bool showSystemRolesM;
     bool showSystemTablesM;
     bool sortDatabasesM;
@@ -127,6 +129,8 @@ void DBHTreeConfigCache::loadFromConfig()
         cfg.get("OrderServersInTree", false));
     // these aren't surfaced by methods, but needed to cause observing tree
     // nodes to update themselves
+    changes += setValue(showSystemIndicesM,
+        cfg.get("ShowSystemIndices", false));
     changes += setValue(showSystemRolesM,
         cfg.get("ShowSystemRoles", false));
     changes += setValue(showSystemTablesM,
@@ -179,6 +183,13 @@ DBHTreeImageList::DBHTreeImageList()
     addImage(ART_Computed);
     addImage(ART_DatabaseConnected);
     addImage(ART_DatabaseDisconnected);
+    addImage(ART_DatabaseServer);
+    addImage(ART_DBTrigger);
+    addImage(ART_DBTriggers);
+    addImage(ART_DDLTrigger);
+    addImage(ART_DDLTriggers);
+    addImage(ART_DMLTrigger);
+    addImage(ART_DMLTriggers);
     addImage(ART_Domain);
     addImage(ART_Domains);
     addImage(ART_Exception);
@@ -188,6 +199,12 @@ DBHTreeImageList::DBHTreeImageList()
     addImage(ART_Functions);
     addImage(ART_Generator);
     addImage(ART_Generators);
+    addImage(ART_GlobalTemporary);
+    addImage(ART_GlobalTemporaries);
+    addImage(ART_Index);
+    addImage(ART_Indices);
+    addImage(ART_Input);
+    addImage(ART_Output);
     addImage(ART_Package);
     addImage(ART_Packages);
     addImage(ART_ParameterInput);
@@ -200,6 +217,8 @@ DBHTreeImageList::DBHTreeImageList()
     addImage(ART_Roles);
     addImage(ART_Root);
     addImage(ART_Server);
+    addImage(ART_SystemIndex);
+    addImage(ART_SystemIndices);
     addImage(ART_SystemDomain);
     addImage(ART_SystemDomains);
     addImage(ART_SystemPackage);
@@ -212,6 +231,10 @@ DBHTreeImageList::DBHTreeImageList()
     addImage(ART_Tables);
     addImage(ART_Trigger);
     addImage(ART_Triggers);
+    addImage(ART_UDF);
+    addImage(ART_UDFs);
+    addImage(ART_User);
+    addImage(ART_Users);
     addImage(ART_View);
     addImage(ART_Views);
 }
@@ -248,6 +271,7 @@ private:
 
     bool nodeVisibleM;
     bool nodeTextBoldM;
+    bool nodeEnabledM;
     wxString nodeTextM;
     int nodeImageIndexM;
     bool showChildrenM;
@@ -264,6 +288,7 @@ public:
     bool getNodeVisible() { return nodeVisibleM; }
     wxString getNodeText() { return nodeTextM; }
     bool getNodeTextBold() { return nodeTextBoldM; }
+    bool getNodeEnabled() { return nodeEnabledM; }
     int getNodeImage() { return nodeImageIndexM; }
     bool getShowChildren() { return showChildrenM; }
     bool getShowNodeExpander() { return showNodeExpanderM; }
@@ -295,8 +320,10 @@ public:
     virtual void visitServer(Server& server);
     virtual void visitTable(Table& table);
     virtual void visitTables(Tables& tables);
+    virtual void visitSysTable(SysTable& table);
     virtual void visitSysTables(SysTables& tables);
-	virtual void visitGTTs(GTTs& tables);
+    virtual void visitGTTable(GTTable& table);
+    virtual void visitGTTables(GTTables& tables);
     virtual void visitDMLTrigger(DMLTrigger& trigger);
     virtual void visitDMLTriggers(DMLTriggers& triggers);
     virtual void visitDBTrigger(DBTrigger& trigger);
@@ -305,15 +332,22 @@ public:
     virtual void visitDDLTriggers(DDLTriggers& triggers);
     virtual void visitUDF(UDF& function);
     virtual void visitUDFs(UDFs& functions);
+    virtual void visitUser(User& user);
+    virtual void visitUsers(Users& users);
     virtual void visitView(View& view);
     virtual void visitViews(Views& views);
+    virtual void visitIndex(Index& index);
+    virtual void visitIndices(Indices& indices);
+    virtual void visitSysIndices(SysIndices& sysIndices);
+    virtual void visitUsrIndices(UsrIndices& UsrIndices);
 };
 
 DBHTreeItemVisitor::DBHTreeItemVisitor(DBHTreeControl* tree)
     : MetadataItemVisitor(), treeM(tree), nodeVisibleM(true),
         nodeTextBoldM(false), nodeTextM(), nodeImageIndexM(-1),
         showChildrenM(false), showNodeExpanderM(false),
-        sortChildrenM(false), nodeConfigSensitiveM(false)
+        sortChildrenM(false), nodeConfigSensitiveM(false),
+        nodeEnabledM(true)
 {
 }
 
@@ -400,13 +434,13 @@ void DBHTreeItemVisitor::visitDatabase(Database& database)
 
 void DBHTreeItemVisitor::visitDomain(Domain& domain)
 {
-    // skip autogenerated domains
+        // skip autogenerated domains
     nodeVisibleM = !domain.isSystem();
     setNodeProperties(&domain, domain.isSystem() ? ART_SystemDomain : ART_Domain);
     if (!nodeVisibleM)
         return;
-    if (domain.propertiesLoaded()) {
         // node text: domain_name + domain_datatype [+ "not null"]
+    if (domain.propertiesLoaded()) {
         nodeTextM = domain.getName_() + " " + domain.getDatatypeAsString();
         if (!domain.isNullable())
         {
@@ -476,7 +510,7 @@ void DBHTreeItemVisitor::visitFunctionSQLs(FunctionSQLs& functions)
 
 void DBHTreeItemVisitor::visitUDF(UDF& function)
 {
-    setNodeProperties(&function, ART_Function);
+    setNodeProperties(&function, ART_UDF);
     if (function.childrenLoaded())
     {
         // make node caption bold when parameter data is loaded
@@ -507,7 +541,17 @@ void DBHTreeItemVisitor::visitUDF(UDF& function)
 
 void DBHTreeItemVisitor::visitUDFs(UDFs& functions)
 {
-    setNodeProperties(&functions, ART_Functions);
+    setNodeProperties(&functions, ART_UDFs);
+}
+
+void DBHTreeItemVisitor::visitUser(User& user)
+{
+    setNodeProperties(&user, ART_User);
+}
+
+void DBHTreeItemVisitor::visitUsers(Users& /*users*/)
+{
+    //setNodeProperties(&users, ART_Users);
 }
 
 void DBHTreeItemVisitor::visitGenerator(Generator& generator)
@@ -558,7 +602,6 @@ void DBHTreeItemVisitor::visitPackage(Package& package)
    
    if (package.childrenLoaded())
     {
-       // jochoa
         // make node caption bold when parameter data is loaded
         // (even if the package has no parameters at all)
         nodeTextBoldM = true;
@@ -675,16 +718,37 @@ void DBHTreeItemVisitor::visitSysTables(SysTables& tables)
     setNodeProperties(&tables, ART_SystemTables); 
 }
 
-void DBHTreeItemVisitor::visitGTTs(GTTs& tables)
+void DBHTreeItemVisitor::visitGTTable(GTTable& table)
 {
-    setNodeProperties(&tables, ART_Tables);
+    setNodeProperties(&table, ART_GlobalTemporary);
+
+    if (table.childrenLoaded())
+    {
+        // make node caption bold when column data has been loaded
+        nodeTextBoldM = true;
+        // show number of columns?
+        if (DBHTreeConfigCache::get().getShowColumnParamCount())
+        {
+            size_t colCount = table.getColumnCount();
+            nodeTextM += wxString::Format(" (%d)", colCount);
+        }
+    }
+    // show Column nodes if Config setting is on
+    showChildrenM = DBHTreeConfigCache::get().getShowColumns();
+    showNodeExpanderM = showChildrenM && !table.childrenLoaded();
+    // update if settings change
+    nodeConfigSensitiveM = true;
+}
+
+void DBHTreeItemVisitor::visitGTTables(GTTables& tables)
+{
+    setNodeProperties(&tables, ART_GlobalTemporaries);
+
 }
 
 void DBHTreeItemVisitor::visitTable(Table& table)
 {
-    setNodeProperties(&table,
-        table.isSystem() ? ART_SystemTable : ART_Table);
-
+    setNodeProperties(&table, ART_Table);
     if (table.childrenLoaded())
     {
         // make node caption bold when column data has been loaded
@@ -708,34 +772,58 @@ void DBHTreeItemVisitor::visitTables(Tables& tables)
     setNodeProperties(&tables, ART_Tables);
 }
 
+void DBHTreeItemVisitor::visitSysTable(SysTable& table)
+{
+    setNodeProperties(&table, ART_SystemTable);
+    if (table.childrenLoaded())
+    {
+        // make node caption bold when column data has been loaded
+        nodeTextBoldM = true;
+        // show number of columns?
+        if (DBHTreeConfigCache::get().getShowColumnParamCount())
+        {
+            size_t colCount = table.getColumnCount();
+            nodeTextM += wxString::Format(" (%d)", colCount);
+        }
+    }
+    // show Column nodes if Config setting is on
+    showChildrenM = DBHTreeConfigCache::get().getShowColumns();
+    showNodeExpanderM = showChildrenM && !table.childrenLoaded();
+    // update if settings change
+    nodeConfigSensitiveM = true;
+}
+
 void DBHTreeItemVisitor::visitDMLTrigger(DMLTrigger& trigger)
 {
-    setNodeProperties(&trigger, ART_Trigger);
+    nodeEnabledM = trigger.getActive();
+    setNodeProperties(&trigger, ART_DMLTrigger);
 }
 
 void DBHTreeItemVisitor::visitDMLTriggers(DMLTriggers& triggers)
 {
-    setNodeProperties(&triggers, ART_Triggers);
+    setNodeProperties(&triggers, ART_DMLTriggers);
 }
 
 void DBHTreeItemVisitor::visitDBTrigger(DBTrigger& trigger)
 {
-    setNodeProperties(&trigger, ART_Trigger);
+    nodeEnabledM = trigger.getActive();
+    setNodeProperties(&trigger, ART_DBTrigger);
 }
 
 void DBHTreeItemVisitor::visitDBTriggers(DBTriggers& triggers)
 {
-    setNodeProperties(&triggers, ART_Triggers); 
+    setNodeProperties(&triggers, ART_DBTriggers); 
 }
 
 void DBHTreeItemVisitor::visitDDLTrigger(DDLTrigger& trigger)
 {
-    setNodeProperties(&trigger, ART_Trigger);
+    nodeEnabledM = trigger.getActive();
+    setNodeProperties(&trigger, ART_DDLTrigger);
 }
 
 void DBHTreeItemVisitor::visitDDLTriggers(DDLTriggers& triggers)
 {
-    setNodeProperties(&triggers, ART_Triggers); 
+    setNodeProperties(&triggers, ART_DDLTriggers); 
 }
 
 void DBHTreeItemVisitor::visitView(View& view)
@@ -764,6 +852,29 @@ void DBHTreeItemVisitor::visitViews(Views& views)
     setNodeProperties(&views, ART_Views);
 }
 
+void DBHTreeItemVisitor::visitIndex(Index& index)
+{
+    nodeEnabledM = index.isActive();
+    
+    //setNodeProperties(&index, index.isSystem() ? ART_SystemIndex : ART_Index);
+    setNodeProperties(&index, ART_Index);
+}
+
+void DBHTreeItemVisitor::visitIndices(Indices& indices)
+{
+    setNodeProperties(&indices, ART_Indices);
+}
+
+void DBHTreeItemVisitor::visitSysIndices(SysIndices& sysIndices)
+{
+    setNodeProperties(&sysIndices, ART_SystemIndices);
+}
+
+void DBHTreeItemVisitor::visitUsrIndices(UsrIndices& usrIndices)
+{
+    setNodeProperties(&usrIndices, ART_Indices);
+}
+
 // TreeSelectionRestorer class
 class TreeSelectionRestorer
 {
@@ -971,7 +1082,6 @@ void DBHTreeItemData::update()
         treeM->SetItemText(id, tivObject.getNodeText());
     if (treeM->GetItemImage(id) != tivObject.getNodeImage())
         treeM->SetItemImage(id, tivObject.getNodeImage());
-
     // track number of visible child nodes for SetItemHasChildren() calls
     unsigned numVisibleChildren = 0;
     // check subitems
@@ -1063,6 +1173,16 @@ void DBHTreeItemData::update()
         // allow for on-demand-loading of children
         treeM->SetItemHasChildren(id, tivObject.getShowNodeExpander());
         treeM->SetItemBold(id, tivObject.getNodeTextBold());
+        if (!tivObject.getNodeEnabled())
+            treeM->SetItemTextColour(id, wxColour(0x080, 0x080, 0x080));
+            //treeM->SetItemTextColour(id, wxSYS_COLOUR_GRAYTEXT);
+        else
+            treeM->SetItemTextColour(id, wxSystemSettings::GetColour(wxSYS_COLOUR_CAPTIONTEXT));
+        
+
+
+
+
         return;
     }
     treeM->SetItemHasChildren(id, true);
@@ -1103,6 +1223,7 @@ void DBHTreeItemData::update()
 
     treeM->SetItemBold(id, tivObject.getNodeTextBold()
         || treeM->ItemHasChildren(id));
+    //treeM->SetBackgroundColour(wxYELLOW);
 }
 
 BEGIN_EVENT_TABLE(DBHTreeControl, wxTreeCtrl)
