@@ -115,8 +115,8 @@ class EventsImpl;
 
 //  Native data types
 typedef enum {ivArray, ivBlob, ivDate, ivTime, ivTimestamp, ivString,
-            ivInt16, ivInt32, ivInt64, ivFloat, ivDouble,
-            ivBool, ivDBKey, ivByte} IITYPE;
+            ivInt16, ivInt32, ivInt64, ivInt128, ivFloat, ivDouble,
+            ivBool, ivDBKey, ivByte, ivDec34, ivDec16} IITYPE;
 
 //
 //  Those are the Interbase C API prototypes that we use
@@ -391,6 +391,7 @@ struct FBCLIENT
 {
     // Attributes
     bool mReady;
+    std::string mfbdll;
 
 #ifdef IBPP_WINDOWS
     HMODULE mHandle;            // The FBCLIENT.DLL HMODULE
@@ -764,6 +765,7 @@ public:
     ServiceImpl(const std::string& ServerName, const std::string& UserName,
                     const std::string& UserPassword);
     ~ServiceImpl();
+    FBCLIENT getGDS() const { return gds; };
 
     //  (((((((( OBJECT INTERFACE ))))))))
 
@@ -807,6 +809,8 @@ class DatabaseImpl : public IBPP::IDatabase
 {
     //  (((((((( OBJECT INTERNALS ))))))))
 
+    FBCLIENT gdsM;	// Local GDS instance
+
     int mRefCount;              // Reference counter
     isc_db_handle mHandle;      // InterBase API Session Handle
     std::string mServerName;    // Server name
@@ -844,6 +848,7 @@ public:
                 const std::string& RoleName, const std::string& CharSet,
                 const std::string& CreateParams);
     ~DatabaseImpl();
+    FBCLIENT getGDS() const { return gds; };
 
     //  (((((((( OBJECT INTERFACE ))))))))
 
@@ -874,6 +879,8 @@ public:
     void Inactivate();
     void Disconnect();
     void Drop();
+    IBPP::IDatabase* Clone();
+
 
     IBPP::IDatabase* AddRef();
     void Release();
@@ -915,6 +922,8 @@ public:
         IBPP::TLR lr = IBPP::lrWait, IBPP::TFF flags = IBPP::TFF(0));
     ~TransactionImpl();
 
+    FBCLIENT getGDS() const { return gds; };
+
     //  (((((((( OBJECT INTERFACE ))))))))
 
 public:
@@ -984,8 +993,11 @@ public:
     void Set(int, int16_t);
     void Set(int, int32_t);
     void Set(int, int64_t);
+    void Set(int, IBPP::ibpp_int128_t);
     void Set(int, float);
     void Set(int, double);
+    void Set(int, IBPP::ibpp_dec16_t);
+    void Set(int, IBPP::ibpp_dec34_t);
     void Set(int, const IBPP::Timestamp&);
     void Set(int, const IBPP::Date&);
     void Set(int, const IBPP::Time&);
@@ -1001,8 +1013,11 @@ public:
     bool Get(int, int16_t&);
     bool Get(int, int32_t&);
     bool Get(int, int64_t&);
+    bool Get(int, IBPP::ibpp_int128_t&);
     bool Get(int, float&);
     bool Get(int, double&);
+    bool Get(int, IBPP::ibpp_dec16_t&);
+    bool Get(int, IBPP::ibpp_dec34_t&);
     bool Get(int, IBPP::Timestamp&);
     bool Get(int, IBPP::Date&);
     bool Get(int, IBPP::Time&);
@@ -1018,6 +1033,7 @@ public:
     bool Get(const std::string&, int16_t&);
     bool Get(const std::string&, int32_t&);
     bool Get(const std::string&, int64_t&);
+    void Get(const std::string&, IBPP::ibpp_int128_t&);
     bool Get(const std::string&, float&);
     bool Get(const std::string&, double&);
     bool Get(const std::string&, IBPP::Timestamp&);
@@ -1082,10 +1098,12 @@ public:
 
     StatementImpl(DatabaseImpl*, TransactionImpl*);
     ~StatementImpl();
+    FBCLIENT getGDS() const { return mDatabase->getGDS(); };
 
     //  (((((((( OBJECT INTERFACE ))))))))
 
 public:
+
     void Prepare(const std::string& sql);
     void Execute(const std::string& sql);
     inline void Execute()   { Execute(std::string()); }
@@ -1107,6 +1125,7 @@ public:
     void Set(int, int16_t);
     void Set(int, int32_t);
     void Set(int, int64_t);
+    void Set(int, IBPP::ibpp_int128_t);
     void Set(int, float);
     void Set(int, double);
     void Set(int, const IBPP::Timestamp&);
@@ -1124,6 +1143,7 @@ public:
     void Set(std::string, int16_t);
     void Set(std::string, int32_t);
     void Set(std::string, int64_t);
+    void Set(std::string, IBPP::ibpp_int128_t);
     void Set(std::string, float);
     void Set(std::string, double);
     void Set(std::string, const IBPP::Timestamp&);
@@ -1154,10 +1174,13 @@ public:
     bool Get(int, int32_t&);
     bool Get(int, int64_t*);
     bool Get(int, int64_t&);
+    bool Get(int, IBPP::ibpp_int128_t&);
     bool Get(int, float*);
     bool Get(int, float&);
     bool Get(int, double*);
     bool Get(int, double&);
+    bool Get(int, IBPP::ibpp_dec16_t&);
+    bool Get(int, IBPP::ibpp_dec34_t&);
     bool Get(int, IBPP::Timestamp&);
     bool Get(int, IBPP::Date&);
     bool Get(int, IBPP::Time&);
@@ -1242,6 +1265,7 @@ public:
     BlobImpl(const BlobImpl&);
     BlobImpl(DatabaseImpl*, TransactionImpl* = 0);
     ~BlobImpl();
+    FBCLIENT getGDS() const { return mDatabase->getGDS(); };
 
     //  (((((((( OBJECT INTERFACE ))))))))
 
@@ -1298,6 +1322,7 @@ public:
     ArrayImpl(const ArrayImpl&);
     ArrayImpl(DatabaseImpl*, TransactionImpl* = 0);
     ~ArrayImpl();
+    FBCLIENT getGDS() const { return mDatabase->getGDS(); };
 
     //  (((((((( OBJECT INTERFACE ))))))))
 
@@ -1386,6 +1411,8 @@ public:
 
     EventsImpl(DatabaseImpl* dbi);
     ~EventsImpl();
+    FBCLIENT getGDS() const { return mDatabase->getGDS(); };
+
 
     //  (((((((( OBJECT INTERFACE ))))))))
 
@@ -1408,9 +1435,13 @@ void decodeDate(IBPP::Date& dt, const ISC_DATE& isc_dt);
 void encodeTime(ISC_TIME& isc_tm, const IBPP::Time& tm);
 void decodeTime(IBPP::Time& tm, const ISC_TIME& isc_tm);
 
+void decodeTimeTz(IBPP::Time& tm, const ISC_TIME_TZ& isc_tm);
+
 void encodeTimestamp(ISC_TIMESTAMP& isc_ts, const IBPP::Timestamp& ts);
 void decodeTimestamp(IBPP::Timestamp& ts, const ISC_TIMESTAMP& isc_ts);
 
+void decodeTimestampTz(IBPP::Timestamp& ts, const ISC_TIMESTAMP_TZ& isc_ts);
+
 struct consts   // See _ibpp.cpp for initializations of these constants
 {
     static const double dscales[19];
